
#!/usr/bin/env python
# -*- coding: utf-8 -*-
from Board import Board
from InputParser import InputParser
from AI import AI
import sys
import random
import Voice
import speech_recognition as sr

import colorama
colorama.init()

from word2number import w2n

WHITE = True
BLACK = False





#%%
'''
   // Voice Recognition (Speech-to-Text) - Google Speech Recognition API
   -> This API converts spoken text (microphone) into written text (Python strings)
   -> Personal or testing purposes only
   -> Generic key is given by default (it may be revoked by Google at any time)
   -> If using API key, quota for your own key is 50 requests per day
'''

#%%

def recognize_speech_from_mic(recognizer, microphone):
    """Transcribe speech from recorded from `microphone`.

    Returns a dictionary with three keys:
    "success": a boolean indicating whether or not the API request was
               successful
    "error":   `None` if no error occured, otherwise a string containing
               an error message if the API could not be reached or
               speech was unrecognizable
    "transcription": `None` if speech could not be transcribed,
               otherwise a string containing the transcribed text
    """
    # check that recognizer and microphone arguments are appropriate type
    if not isinstance(recognizer, sr.Recognizer):
        raise TypeError("`recognizer` must be `Recognizer` instance")

    if not isinstance(microphone, sr.Microphone):
        raise TypeError("`microphone` must be `Microphone` instance")

    # adjust the recognizer sensitivity to ambient noise and record audio
    # from the microphone
    with microphone as source:
        recognizer.adjust_for_ambient_noise(source) # #  analyze the audio source for 1 second
        audio = recognizer.listen(source)

    # set up the response object
    response = {
        "success": True,
        "error": None,
        "transcription": None
    }

    # try recognizing the speech in the recording
    # if a RequestError or UnknownValueError exception is caught,
    #   update the response object accordingly
    try:
        response["transcription"] = recognizer.recognize_google(audio)
    except sr.RequestError:
        # API was unreachable or unresponsive
        response["success"] = False
        response["error"] = "API unavailable/unresponsive"
    except sr.UnknownValueError:
        # speech was unintelligible
        response["error"] = "Unable to recognize speech"

    return response

#%%

# This will recieve the string said by the user
# This function has to convert/filter this string to a specific chess movement
def translate_voice_string_to_chess_command(raw_string):
    
    string_to_parse = raw_string
    
    string_to_parse = string_to_parse.lower() # make the whole string lowercase;

    #temp = re.findall(r'\d+', string_to_parse) # This will get all the different numbers
    #numbers = list(map(int, temp)) 

    # Get the position to be moved to:
    # Look for the space (" ") and get until the end of the string
    # E.g. Pawn a3 -> It should get a3
    try:
        position = string_to_parse[string_to_parse.index(" ")+1:]
    except:
        position = 99999

    king = string_to_parse.find("king")
    queen = string_to_parse.find("queen")
    bishop = string_to_parse.find("bishop")
    knight = string_to_parse.find("knight")
    night = string_to_parse.find("night")
    rook = string_to_parse.find("rook")
    pawn = string_to_parse.find("pawn")
    
    if (king != -1 and position !=99999):
        movement = "K" + position     
    elif (queen != -1 and position !=99999):
        movement = "Q" + position    
    elif (bishop != -1 and position !=99999):
        movement = "B" + position    
    elif ((knight != -1 and position !=99999) or (night != -1 and position !=99999)):
        movement = "N" + position
    elif (rook != -1 and position !=99999):
        movement = "R" + position
    elif (pawn != -1 and position !=99999):
        movement = position
    else:
        movement = "not found"

    return movement

def prepare_xphysical_movement(pos_origin,pos_destiny):
    xaxis_difference = pos_destiny[0] - pos_origin[0]
    
    # Calculating Direction (i.e. if goes right or left)
    if xaxis_difference < 1: # If true, this negative value means that we move to left
        dir = 0 # 0 = left
    else:
        dir = 1 # 1 = right

    dist = abs(xaxis_difference) # Number of squares to move
    axis = 0 # This will move on horizontal

    return axis,dir,dist

def prepare_yphysical_movement(pos_origin,pos_destiny):
    yaxis_difference = pos_destiny[1] - pos_origin[1]
    
    # Calculating Direction (i.e. if goes up or down)
    if yaxis_difference < 1: # If true, this negative value means that we move down
        dir = 0 # 0 = move down
    else:
        dir = 1 # 1 = move up

    dist = abs(yaxis_difference) # Number of squares to move
    axis = 1 # This will move on vertical

    return axis,dir,dist


def askForPlayerSide():
    playerChoiceInput = input(
        "What side would you like to play as [wB]? ").lower()
    if 'w' in playerChoiceInput:
        print("You will play as white")
        return WHITE
    else:
        print("You will play as black")
        return BLACK


def askForDepthOfAI():
    depthInput = 2
    try:
        depthInput = int(input("How deep should the AI look for moves?\n"
                               "Warning : values above 3 will be very slow."
                               " [2]? "))
    except KeyboardInterrupt:
        sys.exit()
    except:
        print("Invalid input, defaulting to 2")
    return depthInput


def printCommandOptions():
    undoOption = 'u : undo last move'
    printLegalMovesOption = 'l : show all legal moves'
    randomMoveOption = 'r : make a random move'
    quitOption = 'quit : resign'
    moveOption = 'a3, Nc3, Qxa2, etc : make the move'
    options = [undoOption, printLegalMovesOption, randomMoveOption,
               quitOption, moveOption, '', ]
    print('\n'.join(options))


def printAllLegalMoves(board, parser):
    for move in parser.getLegalMovesWithNotation(board.currentSide, short=True):
        print(move.notation)


def getRandomMove(board, parser):
    legalMoves = board.getAllMovesLegal(board.currentSide)
    randomMove = random.choice(legalMoves)
    randomMove.notation = parser.notationForMove(randomMove)
    return randomMove


def makeMove(move, board):
    print("Making move : " + move.notation)
    board.makeMove(move)


def printPointAdvantage(board):
    print("Currently, the point difference is : " +
          str(board.getPointAdvantageOfSide(board.currentSide)))


def undoLastTwoMoves(board):
    if len(board.history) >= 2:
        board.undoLastMove()
        board.undoLastMove()


def startGame(board, playerSide, ai):
    parser = InputParser(board, playerSide)
    while True:
        print()
        print(board)
        print()
        if board.isCheckmate():
            if board.currentSide == playerSide:
                print("Checkmate, you lost")
            else:
                print("Checkmate! You won!")
            return

        if board.isStalemate():
            if board.currentSide == playerSide:
                print("Stalemate")
            else:
                print("Stalemate")
            return

        if board.currentSide == playerSide:
            # printPointAdvantage(board)
            move = None
            command = input("It's your move."
                            " Type '?' for options. ? ")


            # Block for getting the voice from the user
            recognizer = sr.Recognizer()
            mic = sr.Microphone(device_index=1)
            print("Start Listening")
            response = recognize_speech_from_mic(recognizer, mic)
            print("After response Listening")
            print('\nSuccess : {}\nError   : {}\n\nText from Speech\n{}\n\n{}' \
                  .format(response['success'],
                          response['error'],
                          '-'*17,
                          response['transcription']))
            full_voice_command = response['transcription']
            print("End Listening")
            # End: Block for getting the voice from the user

            # Make the necessary treatments to get from the voice string the correct movement
            move_to_be_made = translate_voice_string_to_chess_command(full_voice_command)


            if command.lower() == 'u':
                undoLastTwoMoves(board)
                continue
            elif command.lower() == '?':
                printCommandOptions()
                continue
            elif command.lower() == 'l':
                printAllLegalMoves(board, parser)
                continue
            elif command.lower() == 'r':
                move = getRandomMove(board, parser)
            elif command.lower() == 'exit' or command.lower() == 'quit':
                return
            try:
                move = parser.parse(move_to_be_made) # originally this was parser.parse(command)
            except ValueError as error:
                print("%s" % error)
                continue
            makeMove(move, board)

            # Make Luke's part here:
            print(move.oldPos)
            print(move.newPos)
            
            xaxis, xdir, xdist = prepare_xphysical_movement(move.oldPos,move.newPos)
            yaxis, ydir, ydist = prepare_yphysical_movement(move.oldPos,move.newPos)

            print("Here")

        else:
            print("AI thinking...")
            move = ai.getBestMove()
            move.notation = parser.notationForMove(move)
            makeMove(move, board)

def twoPlayerGame(board):
    parserWhite = InputParser(board, WHITE)
    parserBlack = InputParser(board, BLACK)
    while True:
        print()
        print(board)
        print()
        if board.isCheckmate():
            print("Checkmate")
            return

        if board.isStalemate():
            print("Stalemate")
            return

        # printPointAdvantage(board)
        if board.currentSide == WHITE:
            parser = parserWhite
        else:
            parser = parserBlack
        move = None
        command = input("It's your move, {}.".format(board.currentSideRep()) + \
                        " Type '?' for options. ? ")
        

        
        if command.lower() == 'u':
            undoLastTwoMoves(board)
            continue
        elif command.lower() == '?':
            printCommandOptions()
            continue
        elif command.lower() == 'l':
            printAllLegalMoves(board, parser)
            continue
        elif command.lower() == 'r':
            move = getRandomMove(board, parser)
        elif command.lower() == 'exit' or command.lower() == 'quit':
            return
        try:
            move = parser.parse(command)
        except ValueError as error:
            print("%s" % error)
            continue
        makeMove(move, board)


board = Board()

try:
    if len(sys.argv) >= 2 and sys.argv[1] == "--two":
        twoPlayerGame(board)
    else:
        playerSide = askForPlayerSide()
        print()
        aiDepth = askForDepthOfAI()
        opponentAI = AI(board, not playerSide, aiDepth)
        startGame(board, playerSide, opponentAI)
except KeyboardInterrupt:
    sys.exit()
